import logging
from types import SimpleNamespace

from aiohttp import ClientSession, TraceRequestStartParams, TraceRequestEndParams

logger = logging.getLogger("govee-cloud")


async def on_request_start(
    session: ClientSession, context: SimpleNamespace, params: TraceRequestStartParams
) -> None:
    logger.info("making %s request to %s", params.method, params.url)


async def on_request_end(
    session: ClientSession, context: SimpleNamespace, params: TraceRequestEndParams
) -> None:
    data = await params.response.read()
    data = data.decode("utf-8") if data else None
    logger.info(
        "%s request to %s completed with status %s and data %s",
        params.method,
        params.url,
        params.response.status,
        data,
    )



import json
from pathlib import Path
from unittest import IsolatedAsyncioTestCase
from unittest.mock import patch

from aioresponses import aioresponses

from devices.air_purifier.h7126 import H7126
from util.govee_api import GoveeAPI
from util.govee_appliance_api import GoveeApplianceAPI


class TestH7126(IsolatedAsyncioTestCase):
    async def asyncSetUp(self):
        self.api_key = "test-api-key"
        self.govee = GoveeAPI(self.api_key, ignore_request_id=True)
        self.govee_appliance = GoveeApplianceAPI(self.api_key)
        self.device_id = "test-device-id"
        self.device = H7126(self.device_id)
        self.mock_aioresponse = aioresponses()
        self.mock_aioresponse.start()

        # Load test data from JSON file
        test_data_path = Path(__file__).parent / "test_data" / "h7126.json"
        with open(test_data_path, "r") as f:
            self.test_data = json.load(f)

    async def asyncTearDown(self):
        await self.govee.client.close()
        self.mock_aioresponse.stop()

    async def test_update(self):
        mock_response = self.test_data["update"]
        mock_response_custom = self.test_data["update_custom_work_mode"]

        with patch("devices.air_purifier.h7126.logger.warning") as mock_logging:
            self.mock_aioresponse.post(
                "https://openapi.api.govee.com/router/api/v1/device/state",
                status=200,
                payload=mock_response,
            )
            await self.device.update(self.govee)
            self.assertEqual(self.device.online, True)
            self.assertEqual(self.device.power_switch, True)
            self.assertEqual(self.device.work_mode, "Low")
            self.assertEqual(self.device.filter_life, 0)
            self.assertEqual(self.device.air_quality, 6)

            # Check that each warning was called exactly once
            mock_logging.assert_any_call(
                "Found unknown capability type devices.capabilities.unknown"
            )
            mock_logging.assert_any_call("Found unknown instance unknown")

            # Verify there were exactly 2 calls
            self.assertEqual(mock_logging.call_count, 2)

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/state",
            status=200,
            payload=mock_response_custom,
        )
        await self.device.update(self.govee)
        self.assertEqual(self.device.online, True)
        self.assertEqual(self.device.power_switch, True)
        self.assertEqual(self.device.work_mode, "Custom")
        self.assertEqual(self.device.filter_life, 0)
        self.assertEqual(self.device.air_quality, 6)

        with patch(
            "devices.air_purifier.h7126.GoveeAPI.get_device_state"
        ) as mock_get_device_state:
            mock_get_device_state.side_effect = Exception("Test exception")
            await self.device.update(self.govee)
            self.assertEqual(self.device.online, False)

    async def test_str(self):
        expected_device_str = "Name: Smart Air Purifier, SKU: H7126, Device ID: test-device-id, Online: False, Power Switch: False, Work Mode: Sleep, Filter Life: 0, Air Quality: 0"
        device_str = self.device.__str__()
        assert device_str == expected_device_str

    async def test_power_switch(self):
        mock_response_on = self.test_data["power_switch_on"]
        mock_response_off = self.test_data["power_switch_off"]

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_response_on,
        )
        await self.device.turn_on(self.govee)
        self.assertTrue(self.device.power_switch)

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_response_off,
        )
        await self.device.turn_off(self.govee)
        self.assertFalse(self.device.power_switch)

    async def test_work_mode(self):
        mock_response_sleep = self.test_data["sleep_work_mode"]
        mock_response_low = self.test_data["low_work_mode"]
        mock_response_high = self.test_data["high_work_mode"]
        mock_response_custom = self.test_data["custom_work_mode"]

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_response_sleep,
        )
        await self.device.set_work_mode(self.govee, "Sleep")
        self.assertEqual(self.device.work_mode, "Sleep")

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_response_low,
        )
        await self.device.set_work_mode(self.govee, "Low")
        self.assertEqual(self.device.work_mode, "Low")

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_response_high,
        )
        await self.device.set_work_mode(self.govee, "High")
        self.assertEqual(self.device.work_mode, "High")

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_response_custom,
        )
        await self.device.set_work_mode(self.govee, "Custom")
        self.assertEqual(self.device.work_mode, "Custom")

        with self.assertRaises(ValueError):
            await self.device.set_work_mode(self.govee, "Invalid")

    async def test_invalid_capability(self):
        capability = {
            "type": "devices.capabilities.unknown",
            "instance": "workMode",
            "state": {"status": "success"},
            "value": {"workMode": 2, "modeValue": 0},
        }

        with patch("devices.air_purifier.h7126.logger.warning") as mock_logging:
            self.device.parse_response(capability)
            mock_logging.assert_called_once_with(
                "Found unknown capability type devices.capabilities.unknown"
            )

    async def test_rate_limit(self):
        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/state",
            status=429,
        )
        await self.device.update(self.govee)
        self.assertEqual(self.device.online, False)

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control", status=429
        )

        await self.device.turn_on(self.govee)
        self.assertEqual(self.device.online, False)

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control", status=429
        )

        await self.device.turn_off(self.govee)
        self.assertEqual(self.device.online, False)

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control", status=429
        )

        await self.device.set_work_mode(self.govee, "Sleep")
        self.assertEqual(self.device.online, False)



import json
from pathlib import Path
from unittest import IsolatedAsyncioTestCase
from unittest.mock import patch

from aioresponses import aioresponses

from devices.fan.h7102 import H7102
from util.govee_api import GoveeAPI


class TestH7102(IsolatedAsyncioTestCase):
    async def asyncSetUp(self):
        self.api_key = "test-api-key"
        self.govee = GoveeAPI(self.api_key, ignore_request_id=True)
        self.device_id = "test-device-id"
        self.device = H7102(self.device_id)
        self.mock_aioresponse = aioresponses()
        self.mock_aioresponse.start()

        # Load test data from JSON file
        test_data_path = Path(__file__).parent / "test_data" / "h7102.json"
        with open(test_data_path, "r") as f:
            self.test_data = json.load(f)

    async def asyncTearDown(self):
        await self.govee.client.close()
        self.mock_aioresponse.stop()

    async def test_power_switch(self):
        mock_response_on = self.test_data["power_switch_on"]
        mock_response_off = self.test_data["power_switch_off"]

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_response_on,
        )
        await self.device.turn_on(self.govee)
        self.assertTrue(self.device.power_switch)

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_response_off,
        )
        await self.device.turn_off(self.govee)
        self.assertFalse(self.device.power_switch)

    async def test_oscillation(self):
        mock_response_on = self.test_data["oscillation_on"]
        mock_response_off = self.test_data["oscillation_off"]

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_response_on,
        )
        await self.device.toggle_oscillation(self.govee, True)
        self.assertTrue(self.device.oscillation_toggle)

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_response_off,
        )
        await self.device.toggle_oscillation(self.govee, False)
        self.assertFalse(self.device.oscillation_toggle)

    async def test_work_mode(self):
        mock_response_custom = self.test_data["custom_work_mode"]
        mock_response_auto = self.test_data["auto_work_mode"]
        mock_response_sleep = self.test_data["sleep_work_mode"]
        mock_response_nature = self.test_data["nature_work_mode"]
        update = self.test_data["update"]
        mock_response_normal = self.test_data["normal_work_mode"]

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_response_custom,
        )
        await self.device.set_work_mode(self.govee, "Custom")
        self.assertEqual(self.device.work_mode, "Custom")

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_response_auto,
        )
        await self.device.set_work_mode(self.govee, "Auto")
        self.assertEqual(self.device.work_mode, "Auto")

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_response_sleep,
        )
        await self.device.set_work_mode(self.govee, "Sleep")
        self.assertEqual(self.device.work_mode, "Sleep")

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_response_nature,
        )
        await self.device.set_work_mode(self.govee, "Nature")
        self.assertEqual(self.device.work_mode, "Nature")

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/state",
            status=200,
            payload=update,
        )

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_response_normal,
        )
        await self.device.update(self.govee)
        self.assertEqual(self.device.fan_speed, 2)

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/state",
            status=200,
            payload=update,
        )

        await self.device.set_work_mode(self.govee, "Normal")
        self.assertEqual(self.device.work_mode, "Normal")
        self.assertEqual(self.device.fan_speed, 1)

        with self.assertRaises(ValueError):
            await self.device.set_work_mode(self.govee, "Invalid")

    async def test_fan_speed(self):
        mock_response = self.test_data["fan_speed"]

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_response,
        )
        await self.device.set_fan_speed(self.govee, 2)
        self.assertEqual(self.device.fan_speed, 2)

        with self.assertRaises(ValueError):
            await self.device.set_fan_speed(self.govee, 0)
        with self.assertRaises(ValueError):
            await self.device.set_fan_speed(self.govee, 9)

    async def test_update(self):
        mock_response = self.test_data["update"]

        with patch("devices.fan.h7102.logger.warning") as mock_logging:
            self.mock_aioresponse.post(
                "https://openapi.api.govee.com/router/api/v1/device/state",
                status=200,
                payload=mock_response,
            )
            await self.device.update(self.govee)
            self.assertEqual(self.device.online, True)
            self.assertEqual(self.device.power_switch, False)
            self.assertEqual(self.device.oscillation_toggle, False)
            self.assertEqual(self.device.work_mode, "Normal")
            self.assertEqual(self.device.fan_speed, 2)
            mock_logging.assert_called_once_with(
                "Found unknown capability type devices.capabilities.unknown"
            )

        with patch(
            "devices.fan.h7102.GoveeAPI.get_device_state"
        ) as mock_get_device_state:
            mock_get_device_state.side_effect = Exception("Test exception")
            await self.device.update(self.govee)
            self.assertEqual(self.device.online, False)

    async def test_str(self):
        expected_device_str = "Name: Smart Tower Fan, SKU: H7102, Device ID: test-device-id, Online: False, Power Switch: False, Oscillation Toggle: False, Work Mode: Normal, Fan Speed: 1"
        device_str = self.device.__str__()
        assert device_str == expected_device_str

    async def test_invalid_capability(self):
        capability = {
            "type": "devices.capabilities.unknown",
            "instance": "workMode",
            "state": {"status": "success"},
            "value": {"workMode": 2, "modeValue": 0},
        }

        with patch("devices.fan.h7102.logger.warning") as mock_logging:
            self.device.parse_response(capability)
            mock_logging.assert_called_once_with(
                "Found unknown capability type devices.capabilities.unknown"
            )

    async def test_rate_limit(self):
        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/state",
            status=429,
        )
        await self.device.update(self.govee)
        self.assertEqual(self.device.online, False)

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control", status=429
        )

        await self.device.turn_on(self.govee)
        self.assertEqual(self.device.online, False)

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control", status=429
        )

        await self.device.turn_off(self.govee)
        self.assertEqual(self.device.online, False)

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control", status=429
        )

        await self.device.set_work_mode(self.govee, "Sleep")
        self.assertEqual(self.device.online, False)

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control", status=429
        )

        await self.device.toggle_oscillation(self.govee, True)
        self.assertEqual(self.device.online, False)

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control", status=429
        )

        await self.device.set_fan_speed(self.govee, 1)
        self.assertEqual(self.device.online, False)



"""Smart Air Purifier"""

import logging

from devices.device_type import DeviceType
from devices.types.air_purifier import AirPurifier
from util.govee_api import GoveeAPI

logger = logging.getLogger("govee-cloud")


class H7126(AirPurifier):
    def __init__(self, device_id: str):
        work_modes: dict = {
            1: "Sleep",
            2: "Low",
            3: "High",
            4: "Custom",
        }
        sku: str = "H7126"
        device_name: str = "Smart Air Purifier"
        super().__init__(sku, device_id, device_name, work_modes)
        self.device_type: DeviceType = DeviceType.AIR_PURIFIER

    def __str__(self):
        return f"Name: {self.device_name}, SKU: {self.sku}, Device ID: {self.device_id}, Online: {self.online}, Power Switch: {self.power_switch}, Work Mode: {self.work_mode}, Filter Life: {self.filter_life}, Air Quality: {self.air_quality}"

    async def update(self, api: GoveeAPI):
        """
        Update the device state
        :param api: The Govee API
        """
        try:
            state = await api.get_device_state(self.sku, self.device_id)
            capabilities: dict = state["capabilities"]
            super().update(capabilities)
        except Exception as e:
            self.online = False
            logger.error(f"Error updating device state: {e}")



import json
from pathlib import Path
from unittest import IsolatedAsyncioTestCase
from aioresponses import aioresponses

from src.util.govee_api import GoveeAPI


class TestGoveeAPI(IsolatedAsyncioTestCase):
    async def asyncSetUp(self):
        self.api_key = "test-api-key"
        self.govee = GoveeAPI(self.api_key)
        self.mock_aioresponse = aioresponses()
        self.mock_aioresponse.start()

        # Load test data from JSON file
        test_data_path = Path(__file__).parent / "test_data" / "govee_api.json"
        with open(test_data_path, "r") as f:
            self.test_data = json.load(f)

    async def asyncTearDown(self):
        await self.govee.client.close()
        self.mock_aioresponse.stop()

    async def test_get_devices(self):
        # Get response data from the loaded test data
        mock_response = self.test_data["get_devices"]

        # Mock the GET request
        self.mock_aioresponse.get(
            "https://openapi.api.govee.com/router/api/v1/user/devices",
            status=200,
            payload=mock_response,
        )

        # Call the method being tested
        result = await self.govee.get_devices()

        # Verify the results
        self.assertEqual(result, mock_response["data"])

    async def test_get_devices_bad_api_key(self):
        self.mock_aioresponse.get(
            "https://openapi.api.govee.com/router/api/v1/user/devices", status=401
        )

        with self.assertRaises(RuntimeError):
            await self.govee.get_devices()

    async def test_get_devices_rate_limited(self):
        self.mock_aioresponse.get(
            "https://openapi.api.govee.com/router/api/v1/user/devices", status=429
        )

        with self.assertRaises(RuntimeError):
            await self.govee.get_devices()

    async def test_get_device_state(self):
        sku = "H7143"
        device = "52:8B:D4:AD:FC:45:5D:FE"
        uuid = "98e662be-9837-439f-833e-83b5152142f5"

        mock_response = self.test_data["get_device_state"]

        # Mock the GET request
        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/state",
            status=200,
            payload=mock_response,
        )

        # Call the method being tested
        result = await self.govee.get_device_state(sku, device, request_id=uuid)

        # Verify the results
        self.assertEqual(result, mock_response["payload"])

    async def test_get_device_state_invalid_request_id(self):
        sku = "H7143"
        device = "52:8B:D4:AD:FC:45:5D:FE"
        uuid = "invalid-request-id"

        mock_response = self.test_data["get_device_state"]

        # Mock the GET request
        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/state",
            status=200,
            payload=mock_response,
        )

        with self.assertRaises(RuntimeError):
            await self.govee.get_device_state(sku, device, request_id=uuid)

    async def test_get_device_state_bad_api_key(self):
        sku = "H7143"
        device = "52:8B:D4:AD:FC:45:5D:FE"
        uuid = "98e662be-9837-439f-833e-83b5152142f5"

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/state", status=401
        )

        with self.assertRaises(RuntimeError):
            await self.govee.get_device_state(sku, device, request_id=uuid)

    async def test_get_device_state_rate_limited(self):
        sku = "H7143"
        device = "52:8B:D4:AD:FC:45:5D:FE"
        uuid = "98e662be-9837-439f-833e-83b5152142f5"

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/state", status=429
        )

        with self.assertRaises(RuntimeError):
            await self.govee.get_device_state(sku, device, request_id=uuid)

    async def test_control_device_on_off(self):
        sku = "H7143"
        device = "52:8B:D4:AD:FC:45:5D:FE"
        uuid = "98e662be-9837-439f-833e-83b5152142f5"

        capability_off = {
            "type": "devices.capabilities.on_off",
            "instance": "powerSwitch",
            "value": 0,
        }

        capability_on = {
            "type": "devices.capabilities.on_off",
            "instance": "powerSwitch",
            "value": 1,
        }

        mock_response_off = self.test_data["control_device_off"]
        mock_response_on = self.test_data["control_device_on"]

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_response_off,
        )

        result_off = await self.govee.control_device(
            sku, device, capability_off, request_id=uuid
        )

        self.assertEqual(result_off, mock_response_off["capability"])

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_response_on,
        )

        result_on = await self.govee.control_device(
            sku, device, capability_on, request_id=uuid
        )

        self.assertEqual(result_on, mock_response_on["capability"])

    async def test_control_device_toggle(self):
        sku = "H7143"
        device = "52:8B:D4:AD:FC:45:5D:FE"
        uuid = "98e662be-9837-439f-833e-83b5152142f5"

        oscillation_toggle = {
            "type": "devices.capabilities.toggle",
            "instance": "oscillationToggle",
            "value": 0,
        }

        mock_oscillation_toggle = self.test_data["oscillation_toggle"]

        nightlight_toggle = {
            "type": "devices.capabilities.toggle",
            "instance": "nightlightToggle",
            "value": 0,
        }

        mock_nightlight_toggle = self.test_data["nightlight_toggle"]

        air_deflector_toggle = {
            "type": "devices.capabilities.toggle",
            "instance": "airDeflectorToggle",
            "value": 0,
        }

        mock_air_deflector_toggle = self.test_data["air_deflector_toggle"]

        gradient_toggle = {
            "type": "devices.capabilities.toggle",
            "instance": "gradientToggle",
            "value": 0,
        }

        mock_gradient_toggle = self.test_data["gradient_toggle"]

        thermostat_toggle = {
            "type": "devices.capabilities.toggle",
            "instance": "thermostatToggle",
            "value": 0,
        }

        mock_thermostat_toggle = self.test_data["thermostat_toggle"]

        warm_mist_toggle = {
            "type": "devices.capabilities.toggle",
            "instance": "warmMistToggle",
            "value": 0,
        }

        mock_warm_mist_toggle = self.test_data["warm_mist_toggle"]

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_oscillation_toggle,
        )

        result_oscillation_toggle = await self.govee.control_device(
            sku, device, oscillation_toggle, request_id=uuid
        )

        self.assertEqual(
            result_oscillation_toggle, mock_oscillation_toggle["capability"]
        )

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_nightlight_toggle,
        )

        result_nightlight_togglee = await self.govee.control_device(
            sku, device, nightlight_toggle, request_id=uuid
        )

        self.assertEqual(
            result_nightlight_togglee, mock_nightlight_toggle["capability"]
        )

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_air_deflector_toggle,
        )

        result_air_deflector_toggle = await self.govee.control_device(
            sku, device, air_deflector_toggle, request_id=uuid
        )

        self.assertEqual(
            result_air_deflector_toggle, mock_air_deflector_toggle["capability"]
        )

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_gradient_toggle,
        )

        result_gradient_toggle = await self.govee.control_device(
            sku, device, gradient_toggle, request_id=uuid
        )

        self.assertEqual(result_gradient_toggle, mock_gradient_toggle["capability"])

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_thermostat_toggle,
        )

        result_thermostat_toggle = await self.govee.control_device(
            sku, device, thermostat_toggle, request_id=uuid
        )

        self.assertEqual(result_thermostat_toggle, mock_thermostat_toggle["capability"])

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_warm_mist_toggle,
        )

        result_warm_mist_toggle = await self.govee.control_device(
            sku, device, warm_mist_toggle, request_id=uuid
        )

        self.assertEqual(result_warm_mist_toggle, mock_warm_mist_toggle["capability"])

    async def test_control_device_color_setting(self):
        sku = "H7143"
        device = "52:8B:D4:AD:FC:45:5D:FE"
        uuid = "98e662be-9837-439f-833e-83b5152142f5"

        color_rgb_capability = {
            "type": "devices.capabilities.color_setting",
            "instance": "colorRgb",
            "value": 0,
        }

        mock_color_rgb_capability = self.test_data["color_rgb_capability"]

        color_temperature_k_capability = {
            "type": "devices.capabilities.color_setting",
            "instance": "colorTemperatureK",
            "value": 0,
        }

        mock_color_temperature_k_capability = self.test_data[
            "color_temperature_k_capability"
        ]

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_color_rgb_capability,
        )

        result_color_rgb_capability = await self.govee.control_device(
            sku, device, color_rgb_capability, request_id=uuid
        )

        self.assertEqual(
            result_color_rgb_capability, mock_color_rgb_capability["capability"]
        )

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_color_temperature_k_capability,
        )

        result_color_temperature_k_capability = await self.govee.control_device(
            sku, device, color_temperature_k_capability, request_id=uuid
        )

        self.assertEqual(
            result_color_temperature_k_capability,
            mock_color_temperature_k_capability["capability"],
        )

    async def test_control_device_mode(self):
        sku = "H7143"
        device = "52:8B:D4:AD:FC:45:5D:FE"
        uuid = "98e662be-9837-439f-833e-83b5152142f5"

        night_light_scene_capability = {
            "type": "devices.capabilities.mode",
            "instance": "nightlightScene",
            "value": 1,
        }

        mock_night_light_scene_capability = self.test_data[
            "night_light_scene_capability"
        ]

        preset_scene_capability = {
            "type": "devices.capabilities.mode",
            "instance": "presetScene",
            "value": 1,
        }

        mock_preset_scene_capability = self.test_data["preset_scene_capability"]

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_night_light_scene_capability,
        )

        result_night_light_scene_capability = await self.govee.control_device(
            sku, device, night_light_scene_capability, request_id=uuid
        )

        self.assertEqual(
            result_night_light_scene_capability,
            mock_night_light_scene_capability["capability"],
        )

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_preset_scene_capability,
        )

        result_preset_scene_capability = await self.govee.control_device(
            sku, device, preset_scene_capability, request_id=uuid
        )

        self.assertEqual(
            result_preset_scene_capability, mock_preset_scene_capability["capability"]
        )

    async def test_control_device_range(self):
        sku = "H7143"
        device = "52:8B:D4:AD:FC:45:5D:FE"
        uuid = "98e662be-9837-439f-833e-83b5152142f5"

        brightness_capability = {
            "type": "devices.capabilities.range",
            "instance": "brightness",
            "value": 50,
        }

        mock_brightness_capability = self.test_data["brightness_capability"]

        humidity_capability = {
            "type": "devices.capabilities.range",
            "instance": "humidity",
            "value": 50,
        }

        mock_humidity_capability = self.test_data["humidity_capability"]

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_brightness_capability,
        )

        result_brightness_capability = await self.govee.control_device(
            sku, device, brightness_capability, request_id=uuid
        )

        self.assertEqual(
            result_brightness_capability, mock_brightness_capability["capability"]
        )

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_humidity_capability,
        )

        result_humidity_capability = await self.govee.control_device(
            sku, device, humidity_capability, request_id=uuid
        )

        self.assertEqual(
            result_humidity_capability, mock_humidity_capability["capability"]
        )

    async def test_control_device_work_mode(self):
        sku = "H7143"
        device = "52:8B:D4:AD:FC:45:5D:FE"
        uuid = "98e662be-9837-439f-833e-83b5152142f5"

        work_mode_capability = {
            "type": "devices.capabilities.work_mode",
            "instance": "workMode",
            "value": {"workMode": 1, "modeValue": 1},
        }

        mock_work_mode_capability = self.test_data["work_mode_capability"]

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_work_mode_capability,
        )

        result_work_mode_capability = await self.govee.control_device(
            sku, device, work_mode_capability, request_id=uuid
        )

        self.assertEqual(
            result_work_mode_capability, mock_work_mode_capability["capability"]
        )

    async def test_control_segment_color_setting(self):
        sku = "H7143"
        device = "52:8B:D4:AD:FC:45:5D:FE"
        uuid = "98e662be-9837-439f-833e-83b5152142f5"

        segmented_color_rgb_capability = {
            "type": "devices.capabilities.segment_color_setting",
            "instance": "segmentedColorRgb",
            "value": {"segment": [0, 1, 2, 3, 4, 5, 6, 7, 8], "rgb": "0x0000ff"},
        }

        mock_segmented_color_rgb_capability = self.test_data[
            "segmented_color_rgb_capability"
        ]

        segmented_brightness_capability = {
            "type": "devices.capabilities.segment_color_setting",
            "instance": "segmentedBrightness",
            "value": {"segment": [0, 1, 2, 3, 4, 5, 6, 7, 8], "brightness": 50},
        }

        mock_segmented_brightness_capability = self.test_data[
            "segmented_brightness_capability"
        ]

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_segmented_color_rgb_capability,
        )

        result_segmented_color_rgb_capability = await self.govee.control_device(
            sku, device, segmented_color_rgb_capability, request_id=uuid
        )

        self.assertEqual(
            result_segmented_color_rgb_capability,
            mock_segmented_color_rgb_capability["capability"],
        )

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_segmented_brightness_capability,
        )

        result_segmented_brightness_capability = await self.govee.control_device(
            sku, device, segmented_brightness_capability, request_id=uuid
        )

        self.assertEqual(
            result_segmented_brightness_capability,
            mock_segmented_brightness_capability["capability"],
        )

    async def test_control_segment_dynamic_scene(self):
        sku = "H7143"
        device = "52:8B:D4:AD:FC:45:5D:FE"
        uuid = "98e662be-9837-439f-833e-83b5152142f5"

        light_scene_capability = {
            "type": "devices.capabilities.dynamic_scene",
            "instance": "lightScene",
            "value": 4,
        }

        mock_light_scene_capability = self.test_data["light_scene_capability"]

        diy_scene_capability = {
            "type": "devices.capabilities.dynamic_scene",
            "instance": "diyScene",
            "value": 4,
        }

        mock_diy_scene_capability = self.test_data["diy_scene_capability"]

        snapshot_capability = {
            "type": "devices.capabilities.dynamic_scene",
            "instance": "snapshot",
            "value": 4,
        }

        mock_snapshot_capability = self.test_data["snapshot_capability"]

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_light_scene_capability,
        )

        result_light_scene_capability = await self.govee.control_device(
            sku, device, light_scene_capability, request_id=uuid
        )

        self.assertEqual(
            result_light_scene_capability, mock_light_scene_capability["capability"]
        )

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_diy_scene_capability,
        )

        result_diy_scene_capability = await self.govee.control_device(
            sku, device, diy_scene_capability, request_id=uuid
        )

        self.assertEqual(
            result_diy_scene_capability, mock_diy_scene_capability["capability"]
        )

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_snapshot_capability,
        )

        result_snapshot_capability = await self.govee.control_device(
            sku, device, snapshot_capability, request_id=uuid
        )

        self.assertEqual(
            result_snapshot_capability, mock_snapshot_capability["capability"]
        )

    async def test_control_music_setting(self):
        sku = "H7143"
        device = "52:8B:D4:AD:FC:45:5D:FE"
        uuid = "98e662be-9837-439f-833e-83b5152142f5"

        music_mode_capability = {
            "type": "devices.capabilities.music_setting",
            "instance": "musicMode",
            "value": {"musicMode": 25, "sensitivity": 50},
        }

        mock_music_mode_capability = self.test_data["music_mode_capability"]

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_music_mode_capability,
        )

        result_music_mode_capability = await self.govee.control_device(
            sku, device, music_mode_capability, request_id=uuid
        )

        self.assertEqual(
            result_music_mode_capability, mock_music_mode_capability["capability"]
        )

    async def test_control_temperature_setting(self):
        sku = "H7143"
        device = "52:8B:D4:AD:FC:45:5D:FE"
        uuid = "98e662be-9837-439f-833e-83b5152142f5"

        target_temperature_capability = {
            "type": "devices.capabilities.temperature_setting",
            "instance": "targetTemperature",
            "value": {"temperature": 25, "unit": "Celsius"},
        }

        mock_target_temperature_capability = self.test_data[
            "target_temperature_capability"
        ]

        slider_temperature_capability = {
            "type": "devices.capabilities.temperature_setting",
            "instance": "sliderTemperature",
            "value": {"autoStop": 1, "temperature": 25, "unit": "Celsius"},
        }

        mock_slider_temperature_capability = self.test_data[
            "slider_temperature_capability"
        ]

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_target_temperature_capability,
        )

        result_target_temperature_capability = await self.govee.control_device(
            sku, device, target_temperature_capability, request_id=uuid
        )

        self.assertEqual(
            result_target_temperature_capability,
            mock_target_temperature_capability["capability"],
        )

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_slider_temperature_capability,
        )

        result_slider_temperature_capability = await self.govee.control_device(
            sku, device, slider_temperature_capability, request_id=uuid
        )

        self.assertEqual(
            result_slider_temperature_capability,
            mock_slider_temperature_capability["capability"],
        )

    async def test_control_device_invalid_capability(self):
        sku = "H7143"
        device = "52:8B:D4:AD:FC:45:5D:FE"
        uuid = "98e662be-9837-439f-833e-83b5152142f5"

        empty_capability = {}

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=empty_capability,
        )

        with self.assertRaises(ValueError):
            await self.govee.control_device(
                sku, device, empty_capability, request_id=uuid
            )

        invalid_type_capability = {"type": 123}

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=invalid_type_capability,
        )

        with self.assertRaises(ValueError):
            await self.govee.control_device(
                sku, device, invalid_type_capability, request_id=uuid
            )

        no_instance_capability = {"type": "devices.capabilities.on_off"}

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=no_instance_capability,
        )

        with self.assertRaises(ValueError):
            await self.govee.control_device(
                sku, device, no_instance_capability, request_id=uuid
            )

        invalid_instance_capability = {
            "type": "devices.capabilities.on_off",
            "instance": 123,
        }

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=invalid_instance_capability,
        )

        with self.assertRaises(ValueError):
            await self.govee.control_device(
                sku, device, invalid_instance_capability, request_id=uuid
            )

        no_value_capability = {
            "type": "devices.capabilities.on_off",
            "instance": "powerSwitch",
        }

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=no_value_capability,
        )

        with self.assertRaises(ValueError):
            await self.govee.control_device(
                sku, device, no_value_capability, request_id=uuid
            )

        invalid_value_capability = {
            "type": "devices.capabilities.on_off",
            "instance": "powerSwitch",
            "value": "invalid-value",
        }

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=invalid_value_capability,
        )

        with self.assertRaises(ValueError):
            await self.govee.control_device(
                sku, device, invalid_value_capability, request_id=uuid
            )

        invalid_capability_type = {
            "type": "devices.capabilities.test",
            "instance": "powerSwitch",
            "value": 0,
        }

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=invalid_capability_type,
        )

        with self.assertRaises(ValueError):
            await self.govee.control_device(
                sku, device, invalid_capability_type, request_id=uuid
            )

        invalid_instance_type = {
            "type": "devices.capabilities.on_off",
            "instance": "test",
            "value": 0,
        }

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=invalid_instance_type,
        )

        with self.assertRaises(ValueError):
            await self.govee.control_device(
                sku, device, invalid_instance_type, request_id=uuid
            )

    async def test_control_device_bad_uuid(self):
        sku = "H7143"
        device = "52:8B:D4:AD:FC:45:5D:FE"
        invalid_uuid = "invalid-request-id"

        capability_off = {
            "type": "devices.capabilities.on_off",
            "instance": "powerSwitch",
            "value": 0,
        }

        mock_response_off = self.test_data["control_device_off"]

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/control",
            status=200,
            payload=mock_response_off,
        )

        with self.assertRaises(RuntimeError):
            await self.govee.control_device(
                sku, device, capability_off, request_id=invalid_uuid
            )

    async def test_get_dynamic_light_scene(self):
        sku = "H7143"
        device = "52:8B:D4:AD:FC:45:5D:FE"

        invalid_uuid = "invalid-request-id"

        uuid = "98e662be-9837-439f-833e-83b5152142f5"

        mock_response = self.test_data["get_dynamic_light_scene"]

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/scenes",
            status=200,
            payload=mock_response,
        )

        result = await self.govee.get_dynamic_light_scene(sku, device, request_id=uuid)

        self.assertEqual(result, mock_response["payload"])

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/scenes",
            status=200,
            payload=mock_response,
        )

        with self.assertRaises(RuntimeError):
            await self.govee.get_dynamic_light_scene(
                sku, device, request_id=invalid_uuid
            )

    async def test_get_diy_scene(self):
        sku = "H7143"
        device = "52:8B:D4:AD:FC:45:5D:FE"

        invalid_uuid = "invalid-request-id"

        uuid = "98e662be-9837-439f-833e-83b5152142f5"

        mock_response = self.test_data["get_diy_scene"]

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/diy-scenes",
            status=200,
            payload=mock_response,
        )

        result = await self.govee.get_diy_scene(sku, device, request_id=uuid)

        self.assertEqual(result, mock_response["payload"])

        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/diy-scenes",
            status=200,
            payload=mock_response,
        )

        with self.assertRaises(RuntimeError):
            await self.govee.get_diy_scene(sku, device, request_id=invalid_uuid)

    async def test_bad_api_response(self):
        response = {"code": 400}

        self.mock_aioresponse.get(
            "https://openapi.api.govee.com/router/api/v1/user/devices",
            status=200,
            payload=response,
        )

        with self.assertRaises(RuntimeError):
            await self.govee.get_devices()

        response = {"code": 400, "message": "Missing Parameter"}

        self.mock_aioresponse.get(
            "https://openapi.api.govee.com/router/api/v1/user/devices",
            status=200,
            payload=response,
        )

        with self.assertRaises(RuntimeError):
            await self.govee.get_devices()

        response = {"code": 400, "msg": "Missing Parameter"}

        self.mock_aioresponse.get(
            "https://openapi.api.govee.com/router/api/v1/user/devices",
            status=200,
            payload=response,
        )

        with self.assertRaises(RuntimeError):
            await self.govee.get_devices()

        response = {"code": 200, "msg": "Success"}

        self.mock_aioresponse.get(
            "https://openapi.api.govee.com/router/api/v1/user/devices",
            status=200,
            payload=response,
        )

        with self.assertRaises(RuntimeError):
            await self.govee.get_devices()



import logging

import aiohttp

from util import on_request_start, on_request_end

logger = logging.getLogger("govee-cloud")


cmds = ["mode", "turn", "gear"]


async def validate_response(response: aiohttp.ClientResponse):
    if response.status != 200:
        raise RuntimeError(
            f"Request failed with status code {response.status} and message {response.reason}"
        )
    response: dict = await response.json()
    if "code" in response:
        if response["code"] != 200 and response["message"] != "Success":
            logger.error(
                "Request failed with error code %s and message %s",
                response["code"],
                response["message"],
            )
            raise RuntimeError(
                f"Request failed with error code {response['code']} and message {response['message']}"
            )
    elif "status" in response:
        if response["status"] != 200 and response["message"] != "Success":
            logger.error(
                "Request failed with error code %s and message %s",
                response["status"],
                response["message"],
            )
            raise RuntimeError(
                f"Request failed with error code {response['status']} and message {response['message']}"
            )
    else:
        logger.error("Request failed with error code %s", response)
        raise RuntimeError(f"Request failed with error code {response}")


def validate_cmd(cmd: dict) -> bool:
    if cmd["name"] not in cmds:
        logger.error("Command %s is not supported", cmd["name"])
        raise ValueError(f"Command {cmd['name']} is not supported")
    if cmd["name"] == "turn" and cmd["value"] not in ["on", "off"]:
        logger.error("Value %s is not supported for command 'turn'", cmd["value"])
        raise ValueError("Value must be `on` or `off` for command 'turn'")
    if (cmd["name"] == "mode" or cmd["name"] == "gear") and type(
        cmd["value"]
    ) is not int:
        logger.error(
            "Value %s is not supported for command 'mode' or 'gear'", cmd["value"]
        )
        raise ValueError("Value must be an integer for command 'mode' or 'gear'")
    return True


class GoveeApplianceAPI:
    def __init__(self, api_key: str):
        self.api_key = api_key
        self.base_url = "https://developer-api.govee.com"
        self.headers = {
            "Govee-API-Key": self.api_key,
            "Content-Type": "application/json",
        }
        trace_config = aiohttp.TraceConfig()
        trace_config.on_request_start.append(on_request_start)
        trace_config.on_request_end.append(on_request_end)
        self.client = aiohttp.ClientSession(
            base_url=self.base_url,
            headers=self.headers,
            raise_for_status=validate_response,
            trace_configs=[trace_config],
        )

    async def control_device(self, model: str, device: str, cmd: dict) -> dict | None:
        """
        Control an appliance device
        :param model: The model of the appliance to control
        :param device: The device ID of the appliance to control
        :param cmd: The command to send to the appliance
        :return: The data response from the API
        """
        body = {"device": device, "model": model, "cmd": cmd}
        validate_cmd(cmd)
        async with self.client.put(
            "/v1/appliance/devices/control", json=body
        ) as response:
            json = await response.json()
            return json["data"]



import logging

from devices.types.basic_fan import BasicFan
from util.govee_api import GoveeAPI

logger = logging.getLogger("govee-cloud")


class Fan(BasicFan):
    def __init__(
        self,
        sku: str,
        device_id: str,
        device_name: str,
        work_modes: dict,
        min_fan_speed: int,
        max_fan_speed: int,
    ):
        super().__init__(sku, device_id, device_name, work_modes)
        self.oscillation_toggle: bool = False
        self.fan_speed: int = 1
        self.min_fan_speed: int = min_fan_speed
        self.max_fan_speed: int = max_fan_speed
        self.speed_work_mode: str = "Normal"

    def update(self, capabilities: dict):
        for capability in capabilities:
            capability_type: str = capability["type"]
            if capability_type == "devices.capabilities.toggle":
                self.oscillation_toggle = capability["state"]["value"] == 1
            elif capability_type == "devices.capabilities.work_mode":
                self.work_mode = self.work_modes[
                    capability["state"]["value"]["workMode"]
                ]
                self.fan_speed = capability["state"]["value"]["modeValue"]
            else:
                super().update(capability)

    def parse_response(self, response: dict):
        capability_type = response["type"]
        if capability_type == "devices.capabilities.on_off":
            self.power_switch = response["value"] == 1
        elif capability_type == "devices.capabilities.toggle":
            self.oscillation_toggle = response["value"] == 1
        elif capability_type == "devices.capabilities.work_mode":
            self.work_mode = self.work_modes[response["value"]["workMode"]]
            self.fan_speed = response["value"]["modeValue"]
        else:
            logger.warning(f"Found unknown capability type {capability_type}")

    async def toggle_oscillation(self, api: GoveeAPI, oscillation: bool):
        """
        Control the oscillation of the device
        :param api: The Govee API
        :param oscillation: True to turn on oscillation, False to turn off oscillation
        """
        capability = {
            "type": "devices.capabilities.toggle",
            "instance": "oscillationToggle",
            "value": 1 if oscillation else 0,
        }
        try:
            response = await api.control_device(self.sku, self.device_id, capability)
            self.parse_response(response)
        except Exception as e:
            self.online = False
            logger.error(f"Error toggling oscillation: {e}")

    async def set_fan_speed(self, api: GoveeAPI, fan_speed: int):
        """
        Set the fan speed of the device
        :param api: The Govee API
        :param fan_speed: The fan speed to set, must be between self.min_fan_speed and self.max_fan_speed
        """
        if fan_speed < self.min_fan_speed or fan_speed > self.max_fan_speed:
            raise ValueError(
                f"Fan speed must be between {self.min_fan_speed} and {self.max_fan_speed}"
            )

        capability = {
            "type": "devices.capabilities.work_mode",
            "instance": "workMode",
            "value": {"workMode": 1, "modeValue": fan_speed},
        }
        try:
            response = await api.control_device(self.sku, self.device_id, capability)
            self.parse_response(response)
        except Exception as e:
            self.online = False
            logger.error(f"Error setting fan speed: {e}")



import logging

logger = logging.getLogger("govee-cloud")


class Thermometer:
    def __init__(self, sku: str, device_id: str, device_name: str):
        self.sku: str = sku
        self.device_id: str = device_id
        self.device_name: str = device_name
        self.online: bool = False
        self.temperature: float = 0.0
        self.humidity: float = 0.0

    def update(self, capabilities: dict):
        for capability in capabilities:
            capability_type: str = capability["type"]
            if capability_type == "devices.capabilities.online":
                self.online = capability["state"]["value"]
            elif capability_type == "devices.capabilities.property":
                instance = capability["instance"]
                if instance == "sensorTemperature":
                    self.temperature = capability["state"]["value"]
                elif instance == "sensorHumidity":
                    self.humidity = capability["state"]["value"]
                else:
                    logger.warning(f"Found unknown instance {instance}")
                    continue
            else:
                logger.warning(f"Found unknown capability type {capability_type}")



from unittest import IsolatedAsyncioTestCase
from unittest.mock import patch, ANY

import aiohttp
from yarl import URL

from util import on_request_start, on_request_end


class TestUtil(IsolatedAsyncioTestCase):
    async def asyncSetUp(self):
        trace_config = aiohttp.TraceConfig()
        trace_config.on_request_start.append(on_request_start)
        trace_config.on_request_end.append(on_request_end)
        self.client = aiohttp.ClientSession(
            trace_configs=[trace_config],
        )

    async def asyncTearDown(self):
        await self.client.close()

    async def test_trace_config(self):
        with patch("util.logger.info") as mock_logging:
            await self.client.get("https://example.com")
            mock_logging.assert_any_call(
                "making %s request to %s", "GET", URL("https://example.com")
            )

            mock_logging.assert_any_call(
                "%s request to %s completed with status %s and data %s",
                "GET",
                URL("https://example.com"),
                200,
                ANY,
            )



import json
from pathlib import Path
from unittest import IsolatedAsyncioTestCase
from unittest.mock import patch

from aioresponses import aioresponses

from devices.thermometer.h5179 import H5179
from util.govee_api import GoveeAPI
from util.govee_appliance_api import GoveeApplianceAPI


class TestH5179(IsolatedAsyncioTestCase):
    async def asyncSetUp(self):
        self.api_key = "test-api-key"
        self.govee = GoveeAPI(self.api_key, ignore_request_id=True)
        self.govee_appliance = GoveeApplianceAPI(self.api_key)
        self.device_id = "test-device-id"
        self.device = H5179(self.device_id)
        self.mock_aioresponse = aioresponses()
        self.mock_aioresponse.start()

        # Load test data from JSON file
        test_data_path = Path(__file__).parent / "test_data" / "h5179.json"
        with open(test_data_path, "r") as f:
            self.test_data = json.load(f)

    async def asyncTearDown(self):
        await self.govee.client.close()
        self.mock_aioresponse.stop()

    async def test_update(self):
        mock_response = self.test_data["update_response"]

        with patch("devices.thermometer.h5179.log.warning") as mock_logging:
            self.mock_aioresponse.post(
                "https://openapi.api.govee.com/router/api/v1/device/state",
                status=200,
                payload=mock_response,
            )
            await self.device.update(self.govee)
            self.assertEqual(self.device.online, True)
            self.assertEqual(self.device.temperature, 66.74)
            self.assertEqual(self.device.humidity, 50)

            # Check that each warning was called exactly once
            mock_logging.assert_any_call(
                "Found unknown capability type devices.capabilities.unknown"
            )
            mock_logging.assert_any_call("Found unknown instance unknown")

            # Verify there were exactly 2 calls
            self.assertEqual(mock_logging.call_count, 2)

        with patch(
            "devices.thermometer.h5179.GoveeAPI.get_device_state"
        ) as mock_get_device_state:
            mock_get_device_state.side_effect = Exception("Test exception")
            await self.device.update(self.govee)
            self.assertEqual(self.device.online, False)

    async def test_str(self):
        expected_device_str = "Name: Wi-Fi Thermometer, Device ID: test-device-id, Online: False, Temperature: 0.0F, Humidity: 0.0%"
        device_str = self.device.__str__()
        assert device_str == expected_device_str

    async def test_rate_limit(self):
        self.mock_aioresponse.post(
            "https://openapi.api.govee.com/router/api/v1/device/state",
            status=429,
        )
        await self.device.update(self.govee)
        self.assertEqual(self.device.online, False)



from enum import Enum


class DeviceType(Enum):
    LIGHT = "devices.types.light"
    AIR_PURIFIER = "devices.types.air_purifier"
    THERMOMETER = "devices.types.thermometer"
    SOCKET = "devices.types.socket"
    SENSOR = "devices.types.sensor"
    HEATER = "devices.types.heater"
    HUMIDIFIER = "devices.types.humidifier"
    DEHUMIDIFIER = "devices.types.dehumidifier"
    ICE_MAKER = "devices.types.ice_maker"
    AROMA_DIFFUSER = "devices.types.aroma_diffuser"
    BOX = "devices.types.box"
    FAN = "devices.types.fan"



import logging

from util.govee_api import GoveeAPI

logger = logging.getLogger("govee-cloud")


class BasicFan:
    def __init__(self, sku: str, device_id: str, device_name: str, work_modes: dict):
        self.work_modes: dict = work_modes
        self.sku: str = sku
        self.device_id: str = device_id
        self.device_name: str = device_name
        self.online: bool = False
        self.power_switch: bool = False
        self.work_mode: str = self.work_modes[1]
        self.speed_work_mode: str = "Custom"

    def update(self, capability: dict):
        capability_type: str = capability["type"]
        if capability_type == "devices.capabilities.online":
            self.online = capability["state"]["value"]
        elif capability_type == "devices.capabilities.on_off":
            self.power_switch = capability["state"]["value"] == 1
        else:
            logger.warning(f"Found unknown capability type {capability_type}")

    def parse_response(self, response: dict):
        capability_type: str = response["type"]
        if capability_type == "devices.capabilities.on_off":
            self.power_switch = response["value"] == 1
        elif capability_type == "devices.capabilities.work_mode":
            if response["value"]["workMode"] == 2:
                self.work_mode = "Custom"
            else:
                self.work_mode = self.work_modes[response["value"]["modeValue"]]
        else:
            logger.warning(f"Found unknown capability type {capability_type}")

    async def turn_on(self, api: GoveeAPI):
        """
        Turn on the device
        :param api: The Govee API
        """
        capability = {
            "type": "devices.capabilities.on_off",
            "instance": "powerSwitch",
            "value": 1,
        }
        try:
            response = await api.control_device(self.sku, self.device_id, capability)
            self.parse_response(response)
        except Exception as e:
            self.online = False
            logger.error(f"Error turning on device: {e}")

    async def turn_off(self, api: GoveeAPI):
        """
        Turn off the device
        :param api: The Govee API
        """
        capability = {
            "type": "devices.capabilities.on_off",
            "instance": "powerSwitch",
            "value": 0,
        }
        try:
            response = await api.control_device(self.sku, self.device_id, capability)
            self.parse_response(response)
        except Exception as e:
            self.online = False
            logger.error(f"Error turning off device: {e}")

    async def set_work_mode(self, api: GoveeAPI, work_mode: str):
        """
        Set the work mode of the device
        :param api: The Govee API
        :param work_mode: The work mode to set, must be in self.work_mode_dict.values()
        """
        if work_mode not in self.work_modes.values():
            raise ValueError(f"Invalid work mode {work_mode}")

        if work_mode == self.speed_work_mode:
            value = {"workMode": 2, "modeValue": 0}
        else:
            work_mode_key = None
            for key, value in self.work_modes.items():
                if value == work_mode:
                    work_mode_key = key

            value = {"workMode": 1, "modeValue": work_mode_key}

        capability = {
            "type": "devices.capabilities.work_mode",
            "instance": "workMode",
            "value": value,
        }
        try:
            response = await api.control_device(self.sku, self.device_id, capability)
            self.parse_response(response)
        except Exception as e:
            self.online = False
            logger.error(f"Error setting work mode: {e}")
